# -*- coding: utf-8 -*-
import django.db.backends.postgresql.schema as postgresql_schema
from django.apps import AppConfig
from django.conf import settings
from django.db.utils import load_backend
from django.utils.translation import gettext_lazy as _
from pgtrigger import features

from thx.pg_history import migrations
from thx.pg_history.core import register_track_history_models


def patch_schema_editor():
    """
    Patch the schema editor to allow for column types to be altered on
    trigger conditions
    """
    if features.schema_editor():  # pragma: no branch
        for config in settings.DATABASES.values():
            backend = load_backend(config["ENGINE"])
            schema_editor_class = backend.DatabaseWrapper.SchemaEditorClass

            if (
                schema_editor_class
                and issubclass(
                    schema_editor_class,
                    postgresql_schema.DatabaseSchemaEditor,
                )
                and not issubclass(schema_editor_class, migrations.DatabaseSchemaEditorMixin)
            ):
                backend.DatabaseWrapper.SchemaEditorClass = type(
                    "DatabaseSchemaEditor",
                    (migrations.DatabaseSchemaEditorMixin, schema_editor_class),
                    {},
                )


class PgHistoryConfig(AppConfig):
    name = 'thx.pg_history'
    label = 'pg_history'
    verbose_name = _('PostgreSQL History')

    def ready(self):
        patch_schema_editor()
        register_track_history_models()
        super().ready()
