# -*- coding: utf-8 -*-
"""PgHistory tracking wrapper"""
import pghistory
from django.apps import apps
from django.conf import settings
from django.utils.module_loading import import_string

from thx.pg_history.models import HistoryModelMixin
from thx.pg_history.utils import fk_context_tracker


def proxy_model_factory(model_class, module=None):
    """Proxy model factory"""

    class ProxyMeta:
        proxy = True

    return type(
        "%sProxy" % model_class.__name__,
        (model_class,),
        {
            'Meta': ProxyMeta,
            '__module__': module or model_class.__module__
        }
    )


def pghistory_patch_model(model_class):
    model_class.prepare_single_fk_context = HistoryModelMixin.prepare_single_fk_context
    model_class.prepare_fk_context = HistoryModelMixin.prepare_fk_context
    model_class.save = fk_context_tracker(model_class.save)
    return model_class


def track_history(*args, **kwargs):
    """
    pghistory track wrapper. Produce a standardized snapshot label and provides default kwargs.
    If you need customizations, use the pghistory track decorator.
    """
    def _model_wrapper(model_class):
        snaphot = pghistory.Snapshot('snapshot')

        # Monkey patch
        model_class = pghistory_patch_model(model_class)

        # Proxy generator, useful for external packages.
        if kwargs.pop("as_proxy", False):
            history_models_module = apps.app_configs[settings.HISTORY_APP_NAME].models_module.__name__
            model_class = proxy_model_factory(model_class, history_models_module)

        kwargs.setdefault("app_label", settings.HISTORY_APP_NAME)
        kwargs.setdefault("exclude", settings.HISTORY_TRACKED_MODELS_EXCLUDED_FIELDS)
        return pghistory.track(snaphot, *args, **kwargs)(model_class)
    return _model_wrapper


def register_track_history_models():
    for model, kwargs in settings.HISTORY_TRACKED_MODELS.items():
        model = import_string(model)
        track_history(**kwargs)(model)
