# -*- coding: utf-8 -*-
import json

from django.core import serializers
from django.core.exceptions import ObjectDoesNotExist
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import ForeignKey

from thx.pg_history.utils import fk_context_tracker


class HistoryModelMixin:
    """History mixin model."""

    @fk_context_tracker
    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)

    def prepare_single_fk_context(self, instance, field):
        try:
            instance_fk = getattr(instance, field.name, None)
        except ObjectDoesNotExist:
            return {}
        if not instance_fk:
            return {}
        if hasattr(instance_fk, 'password'):
            instance_fk.password = None
        data = serializers.serialize("json", [instance_fk], cls=DjangoJSONEncoder)
        return json.loads(data)[0]

    def prepare_fk_context(self):
        fields = self.__class__._meta.get_fields()
        ctx = {
            field.name: self.prepare_single_fk_context(self, field) for field in fields if isinstance(field, ForeignKey)
        }
        return ctx
